-- ========================================================================== --
-- 										 EskaQuestTracker                                       --
-- @Author   : Skamer <https://mods.curse.com/members/DevSkamer>              --
-- @Website  : https://wow.curseforge.com/projects/eska-quest-tracker         --
-- ========================================================================== --
Scorpio                "EskaQuestTracker.Options.OptionBuilder"               ""
-- ========================================================================== --
namespace "EQT"                                                               --
import "System.Reflector"                                                             --
-- ========================================================================== --
__Flags__()
enum "TextOptionFlags" {
  TEXT_FONT_OPTION = 1,
  TEXT_SIZE_OPTION = 2,
  TEXT_COLOR_OPTION = 4,
  TEXT_TRANSFORM_OPTION = 8,
}

__Unique__()
class "OptionBuilder"

__Arguments__ { Class,
{ Type = String, Nilable = true },
{ Type = TextOptionFlags, Nilable = true, Default = 15} }
function CreateTextOptions(self, class, element, flags)
  local t = {}
  -- Text Size option part
  if ValidateFlags(flags, TextOptionFlags.TEXT_SIZE_OPTION) then
    t.textSize = {
      type = "range",
      name = "Text Size",
      order = 1,
      step = 1,
      min = 4,
      max = 32,
      get = function()
        if element then
          return class:GetTextSize(element)
        else
          return class.textSize
        end
      end,
      set = function(_, value)
        if element then
          class:SetTextSize(element, value)
        else
          class.textSize = value
        end
      end
    }
  end
  -- Text Font option part
  if ValidateFlags(flags, TextOptionFlags.TEXT_FONT_OPTION) then
    t.textFont = {
      type = "select",
      name = "Text Font",
      values = _Fonts,
      itemControl = "DDI-Font",
      get = function()
        if element then
          return GetFontIndex(class:GetTextFont(element))
        else
          return GetFontIndex(class.textFont)
        end
      end,
      set = function(_, value)
        if element then
          class:SetTextFont(element, _Fonts[value])
        else
          class.textFont = _Fonts[value]
        end
      end
    }
  end
  -- Text Color option part
  if ValidateFlags(flags, TextOptionFlags.TEXT_COLOR_OPTION) then
    t.textColor = {
      type = "color",
      name = "Text Color",
      get = function()
        local color
        if element then
          color = class:GetTextColor(element)
        else
          color = class.textColor
        end
        return color.r, color.g, color.b
      end,
      set = function(_, r, g, b, a)
        if element then
          class:SetTextColor(element, { r = r, g = g, b = b })
        else
          class.textColor = { r = r, g = g, b = b }
        end
      end
    }
  end
  -- Text Transform option part
  if ValidateFlags(flags, TextOptionFlags.TEXT_TRANSFORM_OPTION) then
    t.textTransform = {
      type = "select",
      name = "Text Transform",
      values = _TextTransforms,
      get = function()
        if element then
          return class:GetTextTransform(element)
        else
          return class.textTransform
        end
      end,
      set = function(_, value)
        if element then
          class:SetTextTransform(element, value)
        else
          class.textTransform = value
        end
      end
    }
  end
  return t
end
endclass "OptionBuilder"
